@@ -46,9 +46,6 @@ vec_ptype2.integer64.default <- function(x, y, ..., x_arg = "x", y_arg = "y") {
 #' @method vec_ptype2.integer64 vctrs_unspecified
 #' @export
 vec_ptype2.integer64.vctrs_unspecified <- function(x, y, ...) bit64::integer64()
-#' @method vec_ptype2.vctrs_unspecified integer64
-#' @export
-vec_ptype2.vctrs_unspecified.integer64 <- function(x, y, ...) bit64::integer64()
 
 #' @method vec_ptype2.integer64 integer64
 #' @export
