@@ -1,5 +1,11 @@
 context("test-type-list-of")
 
+test_that("list_of inherits from list", {
+  skip("Disabled")
+  x1 <- list_of(1, 1)
+  expect_s3_class(x1, "list")
+})
+
 test_that("list_of works like list", {
   x1 <- list_of(1, 1)
   expect_type(x1, "list")
@@ -181,3 +187,9 @@ test_that("list_of() has as.character() method (tidyverse/tidyr#654)", {
   exp <- rep(paste0("<", vec_ptype_abbr(mtcars), ">"), 2)
   expect_identical(as.character(list_of(mtcars, mtcars)), exp)
 })
+
+test_that("vec_ptype2(<list_of<>>, NA) is symmetric (#687)", {
+  lof <- list_of(1, 2, 3)
+  expect_identical(vec_ptype2(lof, NA), vec_ptype(lof))
+  expect_identical(vec_ptype2(NA, lof), vec_ptype(lof))
+})
